#include "headers/shell.h"
#include "headers/msgQueue.h"

#define MAX_INPUT_LENGTH    50
#define MAX_OUTPUT_LENGTH   100

static const int8_t * const pcWelcomeMessage = 
  "FreeRTOS command server.\r\nType Help to view a list of registered commands.\r\n";

/*static xCommandLineInput cmdToggleLed = {
		.pcCommand = "toggle",
		.pcHelpString = "toggle n: Toggle the LED specified as a parameter n [0 - 3].\r\n",
		.pxCommandInterpreter = prvToggleLed,
		.cExpectedNumberOfParameters = 1
};*/ 

static xCommandLineInput cmdTable[] = {{
											.pcCommand = "toggle",
											.pcHelpString = "toggle n: Toggle the LED specified as a parameter n [0 - 3].\r\n",
											.pxCommandInterpreter = prvToggleLed,
											.cExpectedNumberOfParameters = 1
										}, {
											.pcCommand = "config_rf",
											.pcHelpString = "config_rf [channel] [datarate] [payload size] [power] \r\n",
											.pxCommandInterpreter = prvConfigRF,
											.cExpectedNumberOfParameters = 4
										}, {
											.pcCommand = "send_raw",
											.pcHelpString = "send_raw: send_raw [size] [payload]\r\n",
											.pxCommandInterpreter = prvSendRaw,
											.cExpectedNumberOfParameters = 2
										}, {
											.pcCommand = "recieve_raw",
											.pcHelpString = "recieve_raw: Gets one message from the queue and prints it\r\n",
											.pxCommandInterpreter = prvRecRaw,
											.cExpectedNumberOfParameters = 0
										}};



void registerCommands(){
	int i;
	for(i = 0; i < sizeof(cmdTable)/sizeof(cmdTable[0]);i++){
	//for(i = 0; i < 4;i++){
		FreeRTOS_CLIRegisterCommand(&cmdTable[i]);		
	}
	
}


uint8_t z;

portBASE_TYPE prvConfigRF(int8_t *pcWriteBuffer, size_t xWriteBufferLen, const int8_t *pcCommandString ){
	//"config_rf: Sets RF params for the radio. Use: config_rf [channel] [datarate] [payload size] [power].\r\n",
	volatile portCHAR* param;
	
	uint8_t channel, datarate, payload_size, power;

	//gpio_toggle_pin(LED3_GPIO);
	
	param = FreeRTOS_CLIGetParameter(pcCommandString,1,&z);
	//This function kills it all
	channel = atoi(param);

	param = FreeRTOS_CLIGetParameter(pcCommandString,2,&z);
	//This function kills it all
	datarate = atoi(param);

	param = FreeRTOS_CLIGetParameter(pcCommandString,3,&z);
	//This function kills it all
	payload_size = atoi(param);

	param = FreeRTOS_CLIGetParameter(pcCommandString,4,&z);
	//This function kills it all
	power = atoi(param);
	
	nrf24_setChannel(channel);
	//nrf24_setRF(datarate,power);
	nrf24_setPayloadSize(payload_size);
	
	uint8_t readChannel = nrf24_spiReadRegister(NRF24_REG_05_RF_CH);
	uint8_t readRF = nrf24_spiReadRegister(NRF24_REG_06_RF_SETUP);
	uint8_t readPayloadSize = nrf24_spiReadRegister(NRF24_REG_11_RX_PW_P0);
	
	

	//rawPrint(param,4);
	//channel = atoi(param);
	
	//param = FreeRTOS_CLIGetParameter(pcCommandString,2,&pz);
	//datarate = atoi(param);
	
	//param = FreeRTOS_CLIGetParameter(pcCommandString,3,&pz);
	//payload_size = atoi(param);
	
	//param = FreeRTOS_CLIGetParameter(pcCommandString,4,&pz);
	//power = atoi(param);
	
	volatile int8_t tz = z;
	
	
	
	//sprintf(pcWriteBuffer,"%d %d %d %d \r\n",power,payload_size,datarate,channel);
	//sprintf(pcWriteBuffer,"%d %d %d %d \r\n",42,43,44,45);
	//sprintf(pcWriteBuffer,"Command: %s Param 1: %c%c Size: %d, Add of z: %p\r\n",pcCommandString,*param,*(param+1),tz,&tz);
	//sprintf(pcWriteBuffer,"Command: %s  END\r\n",pcCommandString);
    sprintf(pcWriteBuffer,"Channel: %d Datarate: %d  Payload Size: %d Power: %d END\r\n",readChannel,readRF,readPayloadSize,readRF);
	xWriteBufferLen = strlen(pcWriteBuffer);
	
	return pdFALSE;
}


portBASE_TYPE prvToggleLed(int8_t *pcWriteBuffer, size_t xWriteBufferLen, const int8_t *pcCommandString ){
	uint8_t* param;
	uint8_t* pz;
	param = FreeRTOS_CLIGetParameter(pcCommandString,1,pz);
	switch(*param){
		case '0':
			gpio_toggle_pin(LED0_GPIO);
			break;
		case '1':
			gpio_toggle_pin(LED1_GPIO);
			break;
		case '2':
			gpio_toggle_pin(LED2_GPIO);
			break;
		case '3':
			gpio_toggle_pin(LED3_GPIO);
			break;
		default:  //Wrong parameters
			sprintf(pcWriteBuffer,"%s\r\n","n must be 0 - 3");
			xWriteBufferLen = strlen(pcWriteBuffer);
			return pdFALSE;
		break;
	}
	
	sprintf(pcWriteBuffer,"%s: %c\r\n","OK",*param);
	xWriteBufferLen = strlen(pcWriteBuffer);
	
	return pdFALSE;
}

portBASE_TYPE prvRecRaw(int8_t *pcWriteBuffer, size_t xWriteBufferLen, const int8_t *pcCommandString ){
	nrf_frame_t qm;
	if (xQueueReceive(rxQueue,&qm,0)){
		//Message avaliable
			sprintf(pcWriteBuffer,"MSG: %s SIZE: %d\r\n",qm.content,qm.size);

	} else {
		//message unavaliable
		sprintf(pcWriteBuffer,"NO MESSAGES \r\n");
		
	}
	
	  xWriteBufferLen = strlen(pcWriteBuffer);
	  return pdFALSE;
}



portBASE_TYPE prvSendRaw(int8_t *pcWriteBuffer, size_t xWriteBufferLen, const int8_t *pcCommandString ){
	uint8_t* param;
	uint8_t* pz;
	nrf_frame_t qm;
	
	param = FreeRTOS_CLIGetParameter(pcCommandString,1,pz);
	uint8_t size = atoi(param);
	
	
	param = FreeRTOS_CLIGetParameter(pcCommandString,2,pz);
	param[size-1] = NULL; //Termination
	
	qm.size = size;
	memcpy_ram2ram(&qm.content,param,size);
	
	if( xQueueSendToBack(txQueue,&qm,0)){
		sprintf(pcWriteBuffer,"Queued in Tx\r\n");
		
	} else {
		sprintf(pcWriteBuffer,"ERROR: No avaliable spots in Tx\r\n");
		
	}
	
	/*
	if (!nrf24_setTransmitAddress((uint8_t*)"clie1", 5)) sprintf(pcWriteBuffer,"setTransmitAddress failed \r\n");
    else if (!nrf24_send((uint8_t*) param, size , false)) sprintf(pcWriteBuffer,"send failed \r\n");
	else if (!nrf24_waitPacketSent()) sprintf(pcWriteBuffer,"waitPacketSent failed size: %d message: %s \r\n",size,param);
	else sprintf(pcWriteBuffer,"SENT !\r\n"); 
	*/
	
	xWriteBufferLen = strlen(pcWriteBuffer);
	
	return pdFALSE;
}

void rawPrint(char* s,uint16_t size){
	int i;
	for(i = 0; i < size; i++) {
		putchar(s[i]);
		//TODO: mutex ??
	}
	
}

void vCommandConsoleTask( void *pvParameters )
{
//Peripheral_Descriptor_t xConsole;
int8_t cRxedChar, cInputIndex = 0;
portBASE_TYPE xMoreDataToFollow;
/* The input and output buffers are declared static to keep them off the stack. */
static int8_t pcOutputString[ MAX_OUTPUT_LENGTH ], pcInputString[ MAX_INPUT_LENGTH ];

    /* This code assumes the peripheral being used as the console has already 
    been opened and configured, and is passed into the task as the task
    parameter.  Cast the task parameter to the correct type. */
    //xConsole = ( Peripheral_Descriptor_t ) pvParameters;

    /* Send a welcome message to the user knows they are connected. */
    //FreeRTOS_write( xConsole, pcWelcomeMessage, strlen( pcWelcomeMessage ) );
	//printf("%s",pcWelcomeMessage);
	//xSemaphoreTake(mutexUART,portMAX_DELAY);
	rawPrint(pcWelcomeMessage,strlen(pcWelcomeMessage));
	//xSemaphoreGive(mutexUART);
    for( ;; )
    {
        /* This implementation reads a single character at a time.  Wait in the
        Blocked state until a character is received. */
        //FreeRTOS_read( xConsole, &cRxedChar, sizeof( cRxedChar ) );
        cRxedChar = getchar();
        if( cRxedChar == '\n' )
        {
            /* A newline character was received, so the input command stirng is
            complete and can be processed.  Transmit a line separator, just to 
            make the output easier to read. */
            //FreeRTOS_write( xConsole, "\r\n", strlen( "\r\n" );
			//xSemaphoreTake(mutexUART,portMAX_DELAY);
			rawPrint(pcInputString,strlen(pcInputString));
			putchar('\r');
			putchar('\n');
			//xSemaphoreGive(mutexUART);		
            /* The command interpreter is called repeatedly until it returns 
            pdFALSE.  See the "Implementing a command" documentation for an 
            exaplanation of why this is. */
            do
            {
                /* Send the command string to the command interpreter.  Any
                output generated by the command interpreter will be placed in the 
                pcOutputString buffer. */
                xMoreDataToFollow = FreeRTOS_CLIProcessCommand
                              (     
                                  pcInputString,   /* The command string.*/
                                  pcOutputString,  /* The output buffer. */
                                  MAX_OUTPUT_LENGTH/* The size of the output buffer. */
                              );

                /* Write the output generated by the command interpreter to the 
                console. */
                //FreeRTOS_write( xConsole, pcOutputString, strlen( pcOutputString ) );
				//xSemaphoreTake(mutexUART,portMAX_DELAY);
				rawPrint(pcOutputString,strlen(pcOutputString));
				//xSemaphoreGive(mutexUART);
				//printf("%s",pcOutputString);
            } while( xMoreDataToFollow != pdFALSE );

            /* All the strings generated by the input command have been sent.
            Processing of the command is complete.  Clear the input string ready 
            to receive the next command. */
            cInputIndex = 0;
            memset( pcInputString, 0x00, MAX_INPUT_LENGTH );
        }
        else
        {
            /* The if() clause performs the processing after a newline character
            is received.  This else clause performs the processing if any other
            character is received. */
		
            if( cRxedChar == '\r' )
            {
                /* Ignore carriage returns. */
            }
            else if( cRxedChar == '\b' )
            {
                /* Backspace was pressed.  Erase the last character in the input
                buffer - if there are any. */
                if( cInputIndex > 0 )
                {
                    cInputIndex--;
                    pcInputString[ cInputIndex ] = '\0';
                }
            }
            else
            {
                /* A character was entered.  It was not a new line, backspace
                or carriage return, so it is accepted as part of the input and
                placed into the input buffer.  When a \n is entered the complete
                string will be passed to the command interpreter. */
                if( cInputIndex < MAX_INPUT_LENGTH )
                {
                    pcInputString[ cInputIndex ] = cRxedChar;
                    cInputIndex++;
                }
            }
        }
    }
}        