/**
 * \file
 *
 * \brief USB Standard I/O (stdio) Example
 *
 * Copyright (c) 2011 - 2012 Atmel Corporation. All rights reserved.
 *
 * \asf_license_start
 *
 * \page License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an
 *    Atmel microcontroller product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * \asf_license_stop
 *
 */

/**
 * \mainpage
 *
 * \section intro Introduction
 * This example demonstrates how to configure a C-library Standard
 * I/O interface to the ASF common USB Device CDC service. The initialization
 * routines, along with board and clock configuration constants, illustrate how
 * to perform serial I/O via a Communication Device Class (CDC) device protocol
 *
 * \section files Main Files
 * - stdio_usb_example.c: the example application.
 * - conf_board.h: board configuration
 * - conf_clock.h: board configuration
 * - stdio_usb.h: Common USB CDC Standard I/O Implementation
 * - read.c : System implementation function used by standard library
 * - write.c : System implementation function used by standard library
 *
 * \section example_description Description of the example
 *   - Send message on USB CDC device to a Virtual Com Port.
 *   - Performs echo of any received character
 *
 * \section contactinfo Contact Information
 * For further information, visit
 * <A href="http://www.atmel.com/avr">Atmel AVR</A>.\n
 */

#include <asf.h>
#include <board.h>
#include <sysclk.h>
#include <stdio_usb.h>

#include "headers/nrf24l01.h"
#include "headers/user_interrupts.h"
#include "headers/shell.h"

#include "asf/thirdparty/freertos/freertos-7.0.0/source/include/projdefs.h"
#include "asf/thirdparty/freertos/freertos-7.0.0/source/include/FreeRTOS.h"
#include "asf/thirdparty/freertos/freertos-7.0.0/source/include/task.h"
#include "asf/thirdparty/freertos/freertos-7.0.0/source/include/semphr.h"
#include "asf/thirdparty/freertos/freertos-7.0.0/source/include/queue.h"

#include "headers/msgQueue.h"


#include "libcsp/include/csp/csp.h"




//#define THIS_ADD "brd01"
//#define TARGET_ADD "brd02"

#define THIS_ADD "brd02"
#define TARGET_ADD "brd01"

#define PAYLOAD_SIZE 32

#define SERVER
//#define CLIENT

 
//xSemaphoreHandle semaphoreRadioEvent = NULL;
xSemaphoreHandle semaphoreRadioEvent;
xSemaphoreHandle semaphoreTX;
xSemaphoreHandle semaphoreRX;
xSemaphoreHandle semaphoreMRT; 
 
 uint8_t nrf24_waitPacketSentSemaphores()
 {

	 // Wait for either the Data Sent or Max ReTries flag, signalling the
	 // end of transmission
	 
	 if (xSemaphoreTake(semaphoreTX,50/portTICK_RATE_MS)) {
		 nrf24_spiWriteRegister(NRF24_REG_07_STATUS, NRF24_TX_DS | NRF24_MAX_RT);
		 gpio_toggle_pin(LED1_GPIO);
		 return true;		 
	 }  
	 
	  if (xSemaphoreTake(semaphoreMRT,50/portTICK_RATE_MS)) {
		 nrf24_flushTx();
		 nrf24_spiWriteRegister(NRF24_REG_07_STATUS, NRF24_TX_DS | NRF24_MAX_RT);
		 gpio_toggle_pin(LED2_GPIO);
		 return false; 		 
	 }

	gpio_toggle_pin(LED3_GPIO);
	return false;
	 
}
	 
 
    void vLedTask0(void *pvParameters){
		while(1) {
	    gpio_toggle_pin(LED0_GPIO);
		//xSemaphoreGive(semaphoreRadioEvent);
	    vTaskDelay(100/portTICK_RATE_MS);
		}		
    }
    
    void vLedTask1(void *pvParameters){
	    while(1) {
		//xSemaphoreTake(semaphoreRadioEvent,portMAX_DELAY);
		gpio_toggle_pin(LED1_GPIO);
	    vTaskDelay(100/portTICK_RATE_MS);
		}		
	}
    
    void vLedTask2(void *pvParameters){
	    while(1) {
		gpio_toggle_pin(LED2_GPIO);
	    vTaskDelay(50/portTICK_RATE_MS);
		}		
	}
    
    void vLedTask3(void *pvParameters){
	    while(1) {
		gpio_toggle_pin(LED3_GPIO);
	    vTaskDelay(1000/portTICK_RATE_MS);
		}		
	}
	
	    char s[16];
	
	    void vDave(void *pvParameters){
			while(1) {
			//printf("Hello ? \r\n");
			//scanf("%s",s);
			//printf("Hello");
			//printf("%s\r\n",s);
			char k = getchar();//(char)udi_cdc_getc();
			putchar(k);
			//putchar();
		    vTaskDelay(1000/portTICK_RATE_MS);
			}			
		}


	    void vRadioEvent(void *pvParameters){
			
			nrf_frame_t qm;
			
		    while(1) {
			  	if(xSemaphoreTake(semaphoreRadioEvent,portMAX_DELAY)) {
				    if( gstatus & NRF24_MASK_RX_DR) {
						xSemaphoreGive(semaphoreRX);	
					}
					if( gstatus & NRF24_MASK_TX_DS) {
						xSemaphoreGive(semaphoreTX);
					}
					if( gstatus & NRF24_MASK_MAX_RT) {
						xSemaphoreGive(semaphoreMRT);
					}
				} else {
					//SemaphoreRadioEvent could not be acquierd (SHOULD NEVER END UP HERE)
				}
			}
		}													
									

	    void vRecieve(void *pvParameters){
		    
		    nrf_frame_t qm;
		    
		    while(1) {
			    if (xSemaphoreTake(semaphoreRX,portMAX_DELAY)){
				    nrf24_recv((uint8_t *) &qm.content ,&qm.size);
				    xQueueSendToBack(rxQueue,&qm,0);
			    } else {
				    //Unobtained Semaphore (Should not happen)
			    }
		    }
	    }		
		
		void vSend(void *pvParameters){
			while(1){	    
				nrf_frame_t qm;
				if (xQueueReceive(txQueue,&qm,portMAX_DELAY)){
						
					if (!nrf24_setTransmitAddress((uint8_t*)TARGET_ADD, 5)) //gpio_clr_gpio_pin(LED1_GPIO);
					if (!nrf24_send((uint8_t*) &qm.content, qm.size , false)) //gpio_clr_gpio_pin(LED2_GPIO);
					nrf24_waitPacketSentSemaphores();
					nrf24_powerUpRx();					
				} else {
					//NO MSG TO SEND
						
				}
			}			
		}											
		
		void vServerTask(void *pvParameters){
			while(1) {
			
			//printf("\n Hello Dave : %d \n",c++);
		     //gpio_toggle_pin(LED0_GPIO);
            //printf("waiting...\r\n");

            nrf24_waitAvailable();
            // ping_client sends us an unsigned long containing its timestamp
           
		    unsigned long data;
            uint8_t len = sizeof(data);
            if (!nrf24_recv((uint8_t*)&data, &len)) printf("read failed \r\n");
            // Now send the same data back
            // Need to set the address of the detination each time, since auto-ack changes the TX address

            if (!nrf24_setTransmitAddress((uint8_t*)"clie1", 5)) printf("setTransmitAddress failed \r\n");

            // Send the same data back
            if (! nrf24_send((uint8_t*) &data, sizeof(data) , false)) printf("send failed \r\n");

            if (!nrf24_waitPacketSent()) printf("waitPacketSent failed \r\n");
			
			
			}			
		}

#ifdef SERVER

/**
 * \brief main function
 */
int main (void)
{
    /* Initialize basic board support features.
     * - Initialize system clock sources according to device-specific
     *   configuration parameters supplied in a conf_clock.h file.
     * - Set up GPIO and board-specific features using additional configuration
     *   parameters, if any, specified in a conf_board.h file.
     */
    sysclk_init();
    board_init();

    // Initialize interrupt vector table support.
    irq_initialize_vectors();
	
	init_ext();  
	
    // Enable interrupts
    cpu_irq_enable();

    /* Call a local utility routine to initialize C-Library Standard I/O over
     * a USB CDC protocol. Tunable parameters in a conf_usb.h file must be
     * supplied to configure the USB device correctly.
     */
	
    stdio_usb_init();
	
	//delay_s(5);
	

    nrf24_initRadioSpi();
    if (!nrf24_setChannel(1)) printf("setChannel failed \n");
    if (!nrf24_setThisAddress((uint8_t*)THIS_ADD, 5)) printf("setThisAddress failed \n");
    if (!nrf24_setPayloadSize(PAYLOAD_SIZE)) printf("setPayloadSize failed \n");

    NRF24DataRate dr = NRF24DataRate2Mbps;
    NRF24TransmitPower pr = NRF24TransmitPower0dBm;

    if (!nrf24_setRF(dr, pr)) printf("setRF failed");

	registerCommands();
	initMsgQueues(8,8);
	
	semaphoreRadioEvent = xSemaphoreCreateCounting(1,0);
	semaphoreRX = xSemaphoreCreateCounting(3,0);
	semaphoreTX = xSemaphoreCreateCounting(1,0);
	semaphoreMRT = xSemaphoreCreateCounting(1,0);
	
	    
	xTaskCreate(&vLedTask0, (const signed portCHAR *)"Led Task 0", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+4, NULL);
	xTaskCreate(&vRecieve,  (const signed portCHAR *)"Recieve Task", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+3, NULL);
	xTaskCreate(&vRadioEvent,  (const signed portCHAR *)"Radio Event Processing Task", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+3, NULL);
	xTaskCreate(&vSend,  (const signed portCHAR *)"Send Task", configMINIMAL_STACK_SIZE+8192, NULL, tskIDLE_PRIORITY+3, NULL);
	xTaskCreate(&vCommandConsoleTask, (const signed portCHAR *)"Shell Task", configMINIMAL_STACK_SIZE+1024, NULL, tskIDLE_PRIORITY+2, NULL);
	
	
	
	
	//csp_init(42);
	//csp_buffer_init(1, 32);
	
	vTaskStartScheduler();
	
	
	return 42;
	
	
	
   /*
	int c = 0;

    while (true)
        {
			 printf("\n Hello Dave : %d \n",c++);
		     gpio_toggle_pin(LED0_GPIO);
            printf("waiting...\r\n");

            nrf24_waitAvailable();
            // ping_client sends us an unsigned long containing its timestamp
           
		    unsigned long data;
            uint8_t len = sizeof(data);
            if (!nrf24_recv((uint8_t*)&data, &len)) printf("read failed \r\n");
            // Now send the same data back
            // Need to set the address of the detination each time, since auto-ack changes the TX address

            if (!nrf24_setTransmitAddress((uint8_t*)"clie1", 5)) printf("setTransmitAddress failed \r\n");

            // Send the same data back
            if (! nrf24_send((uint8_t*) &data, sizeof(data) , false)) printf("send failed \r\n");

            if (!nrf24_waitPacketSent()) printf("waitPacketSent failed \r\n");

            //delay_ms(15);
        }
		
*/		
}

#endif  // SERVER
#ifdef CLIENT

int main (void)
{
    /* Initialize basic board support features.
     * - Initialize system clock sources according to device-specific
     *   configuration parameters supplied in a conf_clock.h file.
     * - Set up GPIO and board-specific features using additional configuration
     *   parameters, if any, specified in a conf_board.h file.
     */
	
    sysclk_init();
    board_init();

	//eic_init(EIC_RADIO_1,&nrf24_irq_eic,1);
	//eic_enable_line (EIC_RADIO_1,1);
	 

    // Initialize interrupt vector table support.
    irq_initialize_vectors();

    // Enable interrupts
    cpu_irq_enable();

    /* Call a local utility routine to initialize C-Library Standard I/O over
     * a USB CDC protocol. Tunable parameters in a conf_usb.h file must be
     * supplied to configure the USB device correctly.
     */
    stdio_usb_init();

    delay_s(5);

    printf("Goooood morning sunshine...I'm a client\n");

    nrf24_initRadioSpi();

    if (!nrf24_setChannel(1))  printf("setChannel failed \n\r");
    if (!nrf24_setThisAddress((uint8_t*)"xplan", 5))  printf("setThisAddress failed \r\n");
    //if (!nrf24_setPayloadSize(sizeof(unsigned long))) printf("setPayloadSize failed \r\n");
	if (!nrf24_setPayloadSize(4)) printf("setPayloadSize failed \r\n");

    NRF24DataRate dr = NRF24DataRate2Mbps;
    NRF24TransmitPower pr = NRF24TransmitPower0dBm;

    if (!nrf24_setRF(dr, pr)) printf("setRF failed");

    //printf("\n New Regs \n");

    //nrf24_printRegisters();

    printf("\n Hello Dave \n");

    while (true)
        {
            //printf("send \r\n");

            // Send some data to the server
            if (!nrf24_setTransmitAddress((uint8_t*)"ardui", 5)) printf("setTransmitAddress failed \r\n");
			
			uint16_t time =  cpu_cy_2_ms(Get_sys_count(),sysclk_get_cpu_hz());
		    
			if(!nrf24_send((uint8_t*) time, sizeof(time), false))	printf("send failed \r\n");
			
			if(!nrf24_waitPacketSent()) //printf("waitPacketSent failed \r\n");
			//printf("Packet Aknowledged: %d\r\n", time);
			
			
			//gpio_toggle_pin(LED3_GPIO);
			
			//nrf24_waitAvailable();
			if( nrf24_waitAvailableTimeout(1000)){
				
				//printf("New Packet Recieved \r\n");
			
				uint16_t data;
				uint16_t len = sizeof(data);
				
				if(!nrf24_recv((uint8_t*) &data,&len)) printf("read failed \r\n");
			
				printf("Ping: %d \r\n",cpu_cy_2_ms(Get_sys_count(),sysclk_get_cpu_hz())-time);	
			} else {
			
			//printf("TIMEOUT \r\n");	
			
			}
			
			gpio_toggle_pin(LED0_GPIO);
	        delay_ms(10);
	
	    }
}



#endif // CLIENT
