/*
 * nrf24l01.c
 *
 * Created: 25/03/2013 15:59:49
 *  Author: Jordi
 */


#include <nrf24l01.h>
#include <asf.h>

spi_options_t spi_options_radio =
{
    .baudrate = 8000000, //1000000,
    .bits = 8,
    .modfdis = 1,
    .reg = 0,
    .spck_delay = 0,
    .spi_mode = SPI_MODE_0,
    .stay_act = 1, //TODO: Dubtable
    .trans_delay = 0
};





static void spi_pin_setup()
{

    static const gpio_map_t spi1_gpio_map = {{AVR32_SPI1_MISO_0_0_PIN, AVR32_SPI1_MISO_0_0_FUNCTION}, //MISO
        {AVR32_SPI1_MOSI_0_0_PIN, AVR32_SPI1_MOSI_0_0_FUNCTION}, //MOSI
        {AVR32_SPI1_NPCS_0_0_PIN, AVR32_SPI1_NPCS_0_0_FUNCTION}, //SS0
        {AVR32_SPI1_SCK_0_0_PIN, AVR32_SPI1_SCK_0_0_FUNCTION}    //SCK
    }; 

    gpio_enable_module(spi1_gpio_map,sizeof(spi1_gpio_map)/sizeof(spi1_gpio_map[0]));

}

static void spi_init_module()
{

    sysclk_enable_peripheral_clock(SPI_RADIO_1);
    spi_pin_setup();

    spi_initMaster(SPI_RADIO_1,&spi_options_radio);
    spi_selectionMode(SPI_RADIO_1, 0, 0, 0);

    spi_setupChipReg(SPI_RADIO_1,&spi_options_radio,FOSC0);

    spi_enable(SPI_RADIO_1);
}

uint8_t nrf24_initRadioSpi()
{
    spi_init_module();

    delay_ms(100); //Wait for NRF24L01 POR TODO

    // Clear interrupts
    nrf24_spiWriteRegister(NRF24_REG_07_STATUS, NRF24_RX_DR | NRF24_TX_DS | NRF24_MAX_RT);

    // Make sure we are powered down
    nrf24_powerDown();

    // Flush FIFOs
    nrf24_flushTx();
    nrf24_flushRx();
	
	//nrf24_spiWriteRegister(NRF24_REG_00_CONFIG, NRF24_MASK_RX_DR | NRF24_MASK_MAX_RT | NRF24_DEFAULT_CONFIGURATION); //Enable only TX interrupt
	
	//Activate dynamic payload size
	//nrf24_activate();
	//nrf24_spiWriteRegister(NRF24_REG_1D_FEATURE, NRF24_EN_DPL);
	//nrf24_spiWriteRegister(NRF24_REG_1C_DYNPD, NRF24_DPL_P0 | NRF24_DPL_P1);

    nrf24_powerUpRx();

    return true;

}


uint8_t nrf24_spiCommand(uint8_t command)
{
    uint16_t status;
    spi_selectChip(SPI_RADIO_1,0);
    spi_write(SPI_RADIO_1,command);
    spi_read(SPI_RADIO_1,&status);
    spi_unselectChip(SPI_RADIO_1,0);
    return (uint8_t) status;
}

// Read and write commands
uint8_t nrf24_spiRead(uint8_t command)
{
    uint16_t val;
    spi_selectChip(SPI_RADIO_1,0);
    spi_write(SPI_RADIO_1,command); // Send the address, discard status
    spi_write(SPI_RADIO_1,0x55); //Send a dumy to clock the read out
    spi_read(SPI_RADIO_1,&val); // The MOSI value is ignored, value is read

    spi_unselectChip(SPI_RADIO_1,0);
    return (uint8_t) val;
}

uint8_t nrf24_spiWrite(uint8_t command, uint8_t val)
{
    uint16_t status;
    spi_selectChip(SPI_RADIO_1,0);
    spi_write(SPI_RADIO_1,command);
    spi_read(SPI_RADIO_1,&status);
    spi_write(SPI_RADIO_1,val);
    spi_unselectChip(SPI_RADIO_1,0);
    return (uint8_t) status;
}

void nrf24_spiBurstRead(uint8_t command, uint8_t* dest, uint8_t len)
{
    uint16_t temp;
    spi_selectChip(SPI_RADIO_1,0);
    spi_write(SPI_RADIO_1,command); // Send the start address, discard status
    while (len--)
        {
            spi_write(SPI_RADIO_1,0x55); //Send Dummy to clk out
            spi_read(SPI_RADIO_1, &temp);
            *(dest++) = (uint8_t) temp; //Assign to array and then increment, note post incr
            //dest++ ;//= ((uint8_t) dest) + 1;
        }

    spi_unselectChip(SPI_RADIO_1,0);

    // 300 microsecs for 32 octet payload
}

uint8_t nrf24_spiBurstWrite(uint8_t command, uint8_t* src, uint8_t len)
{
    uint16_t status ;
    spi_selectChip(SPI_RADIO_1,0);

    spi_write(SPI_RADIO_1,command);
    spi_read(SPI_RADIO_1,&status);

    while (len--)
        {
            spi_write(SPI_RADIO_1,*src++);
        }

    spi_unselectChip(SPI_RADIO_1,0);
    return (uint8_t) status;
}


// Use the register commands to read and write the registers
uint8_t nrf24_spiReadRegister(uint8_t reg)
{
    return nrf24_spiRead((reg & NRF24_REGISTER_MASK) | NRF24_COMMAND_R_REGISTER);
}

uint8_t nrf24_spiWriteRegister(uint8_t reg, uint8_t val)
{
    return nrf24_spiWrite((reg & NRF24_REGISTER_MASK) | NRF24_COMMAND_W_REGISTER, val);
}

void nrf24_spiBurstReadRegister(uint8_t reg, uint8_t* dest, uint8_t len)
{
    return nrf24_spiBurstRead((reg & NRF24_REGISTER_MASK) | NRF24_COMMAND_R_REGISTER, dest, len);
}

uint8_t nrf24_spiBurstWriteRegister(uint8_t reg, uint8_t* src, uint8_t len)
{
    return nrf24_spiBurstWrite((reg & NRF24_REGISTER_MASK) | NRF24_COMMAND_W_REGISTER, src, len);
}

uint8_t nrf24_statusRead()
{
    return nrf24_spiReadRegister(NRF24_REG_07_STATUS);
    //    return nrf24_spiCommand(NRF24_COMMAND_NOP); // Side effect is to read status
}

uint8_t nrf24_flushTx()
{
    return nrf24_spiCommand(NRF24_COMMAND_FLUSH_TX);
}

uint8_t nrf24_flushRx()
{
    return nrf24_spiCommand(NRF24_COMMAND_FLUSH_RX);
}

uint8_t nrf24_setChannel(uint8_t channel)
{
    nrf24_spiWriteRegister(NRF24_REG_05_RF_CH, channel & NRF24_RF_CH);
    return true;
}

uint8_t nrf24_setThisAddress(uint8_t* address, uint8_t len)
{
    // Set RX_ADDR_P1 for this address
    nrf24_spiBurstWriteRegister(NRF24_REG_0B_RX_ADDR_P1, address, len);
    // RX_ADDR_P2 is set to RX_ADDR_P1 with the LSbyte set to 0xff, for use as a broadcast address
    return true;
}

uint8_t nrf24_setTransmitAddress(uint8_t* address, uint8_t len)
{
    // Set both TX_ADDR and RX_ADDR_P0 for auto-ack with Enhanced shockwave
    nrf24_spiBurstWriteRegister(NRF24_REG_0A_RX_ADDR_P0, address, len);
    nrf24_spiBurstWriteRegister(NRF24_REG_10_TX_ADDR, address, len);
    return true;
}

uint8_t nrf24_setPayloadSize(uint8_t size)
{
    nrf24_spiWriteRegister(NRF24_REG_11_RX_PW_P0, size);
    nrf24_spiWriteRegister(NRF24_REG_12_RX_PW_P1, size);
    return true;
}

uint8_t nrf24_setRF(uint8_t data_rate, uint8_t power)
{
    uint8_t value = (power << 1) & NRF24_PWR;
    // Ugly mapping of data rates to noncontiguous 2 bits:
    if (data_rate == NRF24DataRate250kbps)
        value |= NRF24_RF_DR_LOW;
    else if (data_rate == NRF24DataRate2Mbps)
        value |= NRF24_RF_DR_HIGH;
    // else NRF24DataRate1Mbps, 00
    nrf24_spiWriteRegister(NRF24_REG_06_RF_SETUP, value);
    return true;
}

uint8_t nrf24_powerDown()
{
    nrf24_spiWriteRegister(NRF24_REG_00_CONFIG, NRF24_DEFAULT_CONFIGURATION);
    gpio_clr_gpio_pin(CE_PIN);
    return true;
}

uint8_t nrf24_powerUpRx()
{
    nrf24_spiWriteRegister(NRF24_REG_00_CONFIG, NRF24_DEFAULT_CONFIGURATION | NRF24_PWR_UP | NRF24_PRIM_RX);
    gpio_set_gpio_pin(CE_PIN);
    return true;
}

uint8_t nrf24_powerUpTx()
{
    // Its the pulse high that puts us into TX mode
    gpio_clr_gpio_pin(CE_PIN);
    nrf24_spiWriteRegister(NRF24_REG_00_CONFIG, NRF24_DEFAULT_CONFIGURATION | NRF24_PWR_UP);
    gpio_set_gpio_pin(CE_PIN);
    return true;
}

uint8_t nrf24_send(uint8_t* data, uint8_t len, uint8_t noack)
{
    nrf24_powerUpTx();
    nrf24_spiBurstWrite(noack ? NRF24_COMMAND_W_TX_PAYLOAD_NOACK : NRF24_COMMAND_W_TX_PAYLOAD, data, len);
    // Radio will return to Standby II mode after transmission is complete
   //delay_ms(10); //TODO: remove delay make it irq driven
	return true;
}

uint8_t nrf24_waitPacketSent()
{
    // If we are currently in receive mode, then there is no packet to wait for
    if (nrf24_spiReadRegister(NRF24_REG_00_CONFIG) & NRF24_PRIM_RX)
        {
            return false;
        }

    // Wait for either the Data Sent or Max ReTries flag, signalling the
    // end of transmission
    uint8_t status;
    while (!((status = nrf24_statusRead()) & (NRF24_TX_DS | NRF24_MAX_RT))) {}


    // Must clear NRF24_MAX_RT if it is set, else no further comm
    nrf24_spiWriteRegister(NRF24_REG_07_STATUS, NRF24_TX_DS | NRF24_MAX_RT);
    if (status & NRF24_MAX_RT)
        {
            nrf24_flushTx();

        }

    // Return true if data sent, false if MAX_RT
	return status & NRF24_TX_DS;
	//return true; //TODO: fer maco
}

uint8_t nrf24_isSending()
{
    return !(nrf24_spiReadRegister(NRF24_REG_00_CONFIG) & NRF24_PRIM_RX) && !(nrf24_statusRead() & (NRF24_TX_DS | NRF24_MAX_RT));
}

uint8_t nrf24_printRegisters()
{
    uint8_t registers[] = { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0d, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x1c, 0x1d};

    uint8_t i;
    for (i = 0; i < sizeof(registers); i++)
        {
            printf("Register: 0x%X: ", registers[i]);
            printf("0x%x \n",nrf24_spiReadRegister(i));
        }
    return true;
}

uint8_t nrf24_available()
{
    if (nrf24_spiReadRegister(NRF24_REG_17_FIFO_STATUS) & NRF24_RX_EMPTY){
	    return false;
		
	}	
    // Manual says that messages > 32 octets should be discarded
    if (nrf24_spiRead(NRF24_COMMAND_R_RX_PL_WID) > 32)
        {
            nrf24_flushRx();
            return false;
        }
    return true;
}

void nrf24_waitAvailable()
{
    nrf24_powerUpRx();
    while (!nrf24_available());
}

// Blocks until a valid message is received or timeout expires
// Return true if there is a message available
uint8_t nrf24_waitAvailableTimeout(uint16_t timeout)
{
    nrf24_powerUpRx();
	t_cpu_time to;
	cpu_set_timeout(cpu_ms_2_cy(timeout,sysclk_get_cpu_hz()),&to);
    
	
    while (! cpu_is_timeout (&to)){
		if (nrf24_available()){
			cpu_stop_timeout(&to);	
			return true;
		}
	}	 
	cpu_stop_timeout(&to);
	return false;
}

uint8_t nrf24_recv(uint8_t* buf, uint8_t* len)
{
    // Clear read interrupt
    nrf24_spiWriteRegister(NRF24_REG_07_STATUS, NRF24_RX_DR);

    // 0 microsecs @ 8MHz SPI clock
    if (!nrf24_available())
        return false;
    // 32 microsecs (if immediately available)
    *len = nrf24_spiRead(NRF24_COMMAND_R_RX_PL_WID);
    // 44 microsecs
    nrf24_spiBurstRead(NRF24_COMMAND_R_RX_PAYLOAD, buf, *len);
    // 140 microsecs (32 octet payload)

    return true;
}

void nrf24_activate(){
	nrf24_spiWrite(NRF24_COMMAND_ACTIVATE,0x73);
}